// Plot the displacements of 1SPP with gap = 0 
clear
clf();
// for the case w1/w2 is rational, choose the eigentvalues of K arbitrarily and find K as following for example: 
A=diag([5^2,1]); 
//X = [2,1;3,1]
K  =[  55.707382  ,  51.36944   ;
      - 32.702721 , - 29.707382  ];
//X=rand(2,2); 
//K=inv(X)*A*X;

//given matrix M,and distance g
m1 = 1.0; m2 = 1.0; 
g = 0.0;

//Find matrix P such that P'MP =I and P'KP = diag(w1^2;w2^2)
M=[m1,0;0,m2];
L=[sqrt(m1),0;0,sqrt(m2)];
Z=inv(L)*K*inv(L');
omega=spec(Z);
[R,diagevals] =spec(Z);

P=inv(L)*R;
iP=inv(P);
C=P'*K*P-diagevals*P'*M*P; // check that P satisfies these conditions <10^-15 is ok

//----------------------------------------------------------
//Define eigenvalues of matrix Omega
w1=sqrt(diagevals(1,1)); tau1=2*%pi/w1;
w2=sqrt(diagevals(2,2)); tau2=2*%pi/w2;

// Define the linear  1dof system during sticking phase 
function u1dot=sys1(t,u1)
    u1dot=P1*u1
endfunction
P1=[0,1;-K(1,1)/m1,0];

// Define the linear 2dof system (~ system of 1st order ODEs)
function udot=sys2(t,u)
         udot=B*u
endfunction
B=       [0,         0,          1,    0;
          0,         0,          0,    1;
    -K(1,1)/m1,   -K(1,2)/m1,    0,    0;
    -K(2,1)/m2,   -K(2,2)/m2,    0,    0];

//finding s such that h(s) = 0 and W1(s) = 0, indeed (w1s,w2s) = (%pi,%pi) mod 2%pi
//by using the theorem of the infinite set Z+ in the paper 1 SPP 





//-----------------------------------------------------------
// Find free flight time s0, sticking time T1 and period T
k= 0 ;
s= (1+2*k)*%pi/w2;
v= 10; //v is arbitrarily positive

m=2;
omega=sqrt(abs(K(1,1))/m1);

T1= ((m-1)*%pi)/omega;

T=T1+s;







//--------------------------------------------------------
//Solve system during free flight time from t0 to T
t0=T1;u0=[g;g;-v;0];
t=T1:0.01:T;
usol=ode(u0,t0,t,sys2);

//Draw the position of the wall
plot(t',g,'gr-')

//draw the time history of displacement
plot2d(t',[usol(1,:)' usol(2,:)'],style=[2,5]);
e=gce();
hl=captions(e.children,['mass 1';'mass 2'],'in_lower_left');

//---------------------------------------------------------
// Solve system during sticking phase where there is only the motion of mass m1
t10=0;u10=[g;v];
t=0:0.01:t0;
u1sol=ode(u10,t10,t,sys1);

//Draw the position of the wall
plot(t',g,'gr-')

//mass m2 stay at the wall during sticking time 0 -> T1
u2sol=g*ones(length(t),1);

//draw the time history of displacement during sticking time 0 ->T1
plot2d(t',[u1sol(1,:)' u2sol],style=[11,5],rect=[0,-15,T,18]);
e=gce();
hl=captions(e.children,['mass 1';'mass 2'],'in_lower_left');


